<?php

/**
 * @file
 * Contains \Drupal\oauth_server_sso\Form\MiniorangeConfigOAuthClient.
 */

namespace Drupal\oauth_server_sso\Form;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\oauth_server_sso\handler;
use Drupal\oauth_server_sso\MiniorangeOAuthServerConstants;
use Drupal\oauth_server_sso\Utilities;


class MiniorangeConfigOAuthClient extends FormBase {

    public function getFormId() {
        return 'oauth_server_sso_config_client';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "oauth_server_sso/oauth_server_sso.style_settings",
                    'oauth_server_sso/oauth_server_sso.moCopy',
                )
            ),
        );

        global $base_url;

        $config = \Drupal::config('oauth_server_sso.settings');
        $status = $config->get('oauth_server_sso_add_client_status');
        $attributes_arr =  array('style' => 'width:90%;');

        $form['mo_oauth_server_style'] = array('#markup' => '<div class="mo_oauth_table_layout_1"><div class="mo_oauth_table_layout mo_oauth_container">');

        if($status == '') {

            $form['markup_top_oauth_client_setup'] = array(
                '#type' => 'fieldset',
                '#title' => t('<h5>OAuth/OIDC Client Setup :</h5>'),
            );

            $form['markup_top_oauth_client_setup']['miniorange_oauth_client_name'] = array(
                '#type' => 'textfield',
                '#default_value' => $config->get('oauth_server_sso_client_name'),
                '#title' => t('Client Name: '),
                '#required' => TRUE,
                '#maxlength' => 1048,
                '#attributes' => $attributes_arr,
                '#prefix' => '<br><hr><br>',
            );

            $form['markup_top_oauth_client_setup']['miniorange_oauth_server_callback_url'] = array(
                '#type' => 'url',
                '#default_value' => $config->get('oauth_server_sso_redirect_url'),
                '#title' => t('Authorized Redirect/Callback URL: '),
                '#required' => TRUE,
                '#maxlength' => 1048,
                '#attributes' => $attributes_arr,
            );

            $form['markup_top_oauth_client_setup']['next_step_1'] = array(
                '#type' => 'submit',
                '#button_type' => 'primary',
                '#prefix' => "<br>",
                '#value' => t('Next'),
                '#submit' => array('::oauth_server_sso_next_1'),
            );

            Utilities::Headless_solution( $form, $form_state);

        } else {

            $form['markup_top_oauth_client_setup'] = array(
                '#type' => 'fieldset',
                '#title' => t('<h5>Summary: </h5>'),
            );

            $form['markup_top_oauth_client_setup']['mo_oauth_server_configuration_table'] = array(
                '#type' => 'table',
                '#attributes' => ['class' => ['mo_oauth_server_configurations']],
                '#empty' => t('Something is not right. Please run the update script or contact us at <a href="mailto:'.MiniorangeOAuthServerConstants::SUPPORT_EMAIL.'">'. MiniorangeOAuthServerConstants::SUPPORT_EMAIL .'</a>'),
                '#responsive' => TRUE ,
                '#prefix' => '<br><hr><br>',
            );

            $client_id = $config->get('oauth_server_sso_client_id');
            $client_secret = $config->get('oauth_server_sso_client_secret');

            $configData = [
                'Client Name' => 'oauth_server_sso_client_name',
                'Redirect/Callback URL' => 'oauth_server_sso_redirect_url',
                'Client ID' => $client_id,
                'Client Secret' => $client_secret
            ];
            foreach ($configData as $key => $value) {
                $row = self::miniorange_oauth_server_configuration($key, $value, $attributes_arr);
                $form['markup_top_oauth_client_setup']['mo_oauth_server_configuration_table'][$key] = $row;
            }

            $form['markup_top_oauth_client_setup']['oauth_server_sso_delete_client'] = array(
                '#type' => 'submit',
                '#value' => t('Delete Client'),
                '#button_type' => 'danger',
                '#limit_validation_errors' => [],
                '#prefix' => '<br>',
                '#submit' => array('::oauth_server_sso_delete_client'),
            );

            $form['markup_top_oauth_client_setup']['next_step_1'] = array(
                '#type' => 'submit',
                '#value' => t('Update'),
                '#button_type' => 'primary',
                '#submit' => array('::oauth_server_sso_next_2'),
            );

            $form['markup_top_oauth_server_endpoints'] = array(
                '#type' => 'fieldset',
                '#title' => t('<h5>Endpoint URLs: </h5>'),
            );

            $form['markup_top_oauth_server_endpoints']['mo_oauth_server_attrs_list_idp'] = array(
                '#type' => 'table',
                '#header'=> array( 'ATTRIBUTE', 'VALUE', '' ),
                '#attributes' => ['class' => ['mo_oauth_server_endpoints']],
                '#empty' => t('Something is not right. Please run the update script or contact us at <a href="mailto:'.MiniorangeOAuthServerConstants::SUPPORT_EMAIL.'">'. MiniorangeOAuthServerConstants::SUPPORT_EMAIL .'</a>'),
                '#responsive' => TRUE ,
                '#prefix' => '<br><hr><br>',
            );

            $data = [
                'Scope' => 'profile openid',
                'Authorize Endpoint' => $base_url.'/authorize',
                'Access Token Endpoint' => $base_url.'/access_token',
                'Get User Info Endpoint' => $base_url.'/user_info'
            ];

            foreach ($data as $key => $value) {
                $row = self::miniorange_oauth_server_endpoints($key, $value);
                $form['markup_top_oauth_server_endpoints']['mo_oauth_server_attrs_list_idp'][$key] = $row;
            }

            Utilities::Headless_solution( $form, $form_state);

            $form['mo_add_oauth_server_div_end'] = array('#markup' => '</div>');

        }
      Utilities::moOAuthServerShowCustomerSupportIcon($form, $form_state);

        return $form;
    }

    public function miniorange_oauth_server_endpoints($endpoint_name, $endpoint_value){
        $row[$endpoint_name] = [
            '#markup' => '<div class="container-inline"><strong>'. $endpoint_name . '</strong>',
        ];

        $row[$endpoint_value] = [
            '#markup' => '<span id="copy_'.str_replace(' ', '_', $endpoint_name).'">'.$endpoint_value.'</span>',
        ];

        $row['copy_'.$endpoint_name] = array(
            '#value' => t('&#128461; Copy'),
            '#type' => 'button',
            '#id' => 'copy_button_'.str_replace(' ', '_', $endpoint_name),
            '#attributes' => ['onclick' => 'CopyToClipboard(copy_'.str_replace(' ', '_', $endpoint_name).')', 'class' => ['use-ajax']],
            '#ajax' => ['event' => 'click',
                'progress' => [
                    'type' => 'throbber',
                    'message' => $this->t('Copying'),
                ]
            ],
            '#suffix' => '</div>',
        );

        return $row;
    }

    public function miniorange_oauth_server_configuration($attr_name, $attr_value, $attributes_arr){
        $config = \Drupal::config('oauth_server_sso.settings');
        $row[$attr_name] = [
            '#markup' => '<div class="container-inline"><strong>'. $attr_name . '</strong>',
        ];

        if($attr_name == 'Client ID' || $attr_name == 'Client Secret'){
            $row[$attr_value] = [
                '#markup' => '<span id="copy_'.str_replace(' ', '_', $attr_name).'">'.$attr_value.'</span>',
            ];

            $row['copy_'.$attr_name] = array(
                '#value' => t('&#128461; Copy'),
                '#type' => 'button',
                '#id' => 'copy_button_'.str_replace(' ', '_', $attr_value),
                '#attributes' => ['onclick' => 'CopyToClipboard(copy_'.str_replace(' ', '_', $attr_name).')', 'class' => ['use-ajax']],
                '#ajax' => ['event' => 'click',
                    'progress' => [
                        'type' => 'throbber',
                        'message' => $this->t('Copying'),
                    ]
                ],
                '#suffix' => '</div>',
            );
        }elseif ($attr_value == 'oauth_server_sso_client_name'){
            $row[$attr_value] = [
                '#type' => 'textfield',
                '#required' => true,
                '#default_value' => $config->get($attr_value),
                '#disabled' => TRUE,
                '#attributes' => $attributes_arr,
                '#suffix' => '</div>'
            ];
        }else{
            $row[$attr_value] = [
                '#type' => 'textfield',
                '#required' => true,
                '#attributes' => $attributes_arr,
                '#default_value' => $config->get($attr_value),
                '#suffix' => '</div>'
            ];
        }

        return $row;
    }

    function oauth_server_sso_next_1(array &$form, FormStateInterface $form_state)
    {
        $form_values = $form_state->getValues();
        $configFactory = \Drupal::configFactory()->getEditable('oauth_server_sso.settings');
        $client_name = $form_values['miniorange_oauth_client_name'];
        $redirect_url = $form_values['miniorange_oauth_server_callback_url'];

        $configFactory->set('oauth_server_sso_client_name',$client_name)->save();
        $configFactory->set('oauth_server_sso_redirect_url',$redirect_url)->save();

        $client_id = handler::generateRandom(30);
        $client_secret = handler::generateRandom(30);
        $configFactory->set('oauth_server_sso_client_id', $client_id)->save();
        $configFactory->set('oauth_server_sso_client_secret', $client_secret)->save();
        $configFactory->set('oauth_server_sso_add_client_status', 'review')->save();

        \Drupal::messenger()->addStatus($this->t('Configurations saved successfully.'));
    }

    function oauth_server_sso_next_2(array &$form, FormStateInterface $form_state)
    {
        $form_values = $form_state->getValues();
        $callback_url = $form_values['mo_oauth_server_configuration_table']['Redirect/Callback URL']['oauth_server_sso_redirect_url'];

        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_redirect_url',$callback_url)->save();
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_add_client_status','review')->save();
        \Drupal::messenger()->addMessage($this->t('Configurations saved successfully.'));
    }

    function oauth_server_sso_delete_client(array &$form, FormStateInterface $form_state)
    {
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('oauth_server_sso_client_name')->save();
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('oauth_server_sso_client_id')->save();
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('oauth_server_sso_client_secret')->save();
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('oauth_server_sso_redirect_url')->save();
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('oauth_server_sso_add_client_status')->save();
    }

    function submitForm(array &$form, FormStateInterface $form_state){}



    /**
     * Send support query.
     */
    function send_support_query(&$form, $form_state)
    {
        Utilities::send_query($form, $form_state);
    }
}
