<?php

/**
 * @file
 * Contains \Drupal\oauth_server_sso\Form\MiniorangeGeneralSettings.
 */
namespace Drupal\oauth_server_sso\Form;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\oauth_server_sso\Utilities;
use Drupal\oauth_server_sso\MiniorangeOAuthServerSupport;

class MiniorangeGeneralSettings extends FormBase
{
    /**
     * {@inheritdoc}
     */
    public function getFormId() {
        return 'miniorange_general_settings';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        global $base_url;

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "oauth_server_sso/oauth_server_sso.style_settings",
                )
            ),
        );

        $url_path = $base_url . '/' . \Drupal::service('extension.list.module')->getPath('oauth_server_sso'). '/includes/images';

        $form['mo_oauth_server_style'] = array('#markup' => '<div class="mo_oauth_table_layout_1"><div class="mo_oauth_table_layout mo_oauth_container">');

        $form['markup_top_oauth_server_general_settings'] = array(
            '#type' => 'fieldset',
            '#title' => t('GENERAL SETTINGS <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium"><span class="mo_pro_tooltip">Available in the Premium version</span></a>'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $form['markup_top_oauth_server_general_settings']['oauth_server_sso_accesstoken_expiry'] = array(
            '#type' => 'textfield',
            '#title' => t('Access Token Expiry Time:'),
            '#disabled' => true,
            '#attributes' => array('placeholder' => 'in seconds'),
            '#prefix' => '<br><hr><br>'
        );

        $form['markup_top_oauth_server_general_settings']['oauth_server_sso_refreshtoken_expiry'] = array(
            '#type' => 'textfield',
            '#title' => t('Refresh Token Expiry Time:'),
            '#disabled' => true,
            '#attributes' => array('placeholder' => 'in seconds'),
            '#suffix' => '<br>',
        );

        $form['markup_top_oauth_server_general_settings']['oauth_server_sso_enforce_state_parameter'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#title' => t('Enforce State Parameter '),
            '#description' => t('When enabled, the authorization request will fail if state parameter is not provided or is incorrect.'),
        );

        $form['markup_top_oauth_server_general_settings']['oauth_server_sso_enable_callback_validation'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#title' => t('Callback/Redirect URL Validation '),
            '#description' => t('By default, server is configured with default redirect URL.<br> You can use this feature incase you have Dynamic or Conditional Callback/Redirect URIs and which to redirect back to different pages'),
        );


        $form['markup_top_oauth_server_grant_support'] = array(
            '#type' => 'fieldset',
            '#title' => t('Allowed Grant Types <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium"><span class="mo_pro_tooltip">Available in the Premium version</span></a>'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $form['markup_top_oauth_server_grant_support']['oauth_server_sso_enable_authorization'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#default_value' => 1,
            '#prefix' => '<br><hr><br>',
            '#title' => t('Authorization Grant'),
        );
        $form['markup_top_oauth_server_grant_support']['oauth_server_sso_enable_implicit'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#title' => t('Implicit Grant'),
        );
        $form['markup_top_oauth_server_grant_support']['oauth_server_sso_enable_password'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#title' => t('Password Grant'),
        );
        $form['markup_top_oauth_server_grant_support']['oauth_server_sso_enable_client'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#title' => t('Client Credentials Grant'),
        );
        $form['markup_top_oauth_server_grant_support']['oauth_server_sso_enable_refresh'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#title' => t('Refresh Token Grant'),
        );

        $form['markup_top_oauth_server_troubleshoot'] = array(
            '#type' => 'fieldset',
            '#title' => t('Debug / Troubleshoot <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium"><span class="mo_pro_tooltip">Available in the Premium version</span></a>'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $form['markup_top_oauth_server_troubleshoot']['oauth_server_sso_enable_loggers'] = array(
            '#type' => 'checkbox',
            '#disabled' => true,
            '#title' => t('Enable logging'),
            '#prefix' => '<br><hr><br>',
            '#description' => t('Enabling this checkbox will add loggers under the <a href="'.$base_url.'/admin/reports/dblog" target="_blank">Reports</a> section'),
        );

        $form['mo_oauth_server_save_general_settings'] = array(
            '#type' => 'submit',
            '#disabled' => true,
            '#value' => t('Save Settings'),
        );




        Utilities::Advertise_2fa( $form, $form_state);
      Utilities::moOAuthServerShowCustomerSupportIcon($form, $form_state);
        return $form;
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {

    }

    /**
     * Send support query.
     */
    function send_support_query(&$form, $form_state)
    {
        Utilities::send_query($form, $form_state);
    }
}
