<?php

/**
 * @file
 * Contains \Drupal\oauth_server_sso\Form\MiniorangeServerMapping.
 */
namespace Drupal\oauth_server_sso\Form;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\oauth_server_sso\MiniorangeOAuthServerConstants;
use Drupal\oauth_server_sso\Utilities;
use Drupal\oauth_server_sso\MiniorangeOAuthServerSupport;

class MiniorangeServerMapping extends FormBase
{
    /**
     * {@inheritdoc}
     */
    public function getFormId() {
        return 'miniorange_general_settings';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        global $base_url;

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "oauth_server_sso/oauth_server_sso.style_settings",
                )
            ),
        );

        $url_path = $base_url . '/' . \Drupal::service('extension.list.module')->getPath('oauth_server_sso'). '/includes/images';

        $form['mo_oauth_server_style'] = array('#markup' => '<div class="mo_oauth_table_layout_1"><div class="mo_oauth_table_layout mo_oauth_container">');


        $form['oauth_server_sso_mapping'] = array(
            '#markup' => t('<h2> Mapping </h2><hr><br><div class="mo_oauth_server_highlight_background_note_2"> <b>NOTE:</b> In this tab, you can view and configure the data that you wish to send from Drupal to your OAuth/OIDC Client.</div>'),
        );

        $form['oauth_server_basic_attribute_mapping'] = array(
            '#type' => 'fieldset',
            '#title' => t('Basic Mapping'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $form['oauth_server_basic_attribute_mapping']['oauth_server_sso_server_response_markup'] = array(
            '#markup' => t('<div class="mo_oauth_server_highlight_background_note_2">In the below section, <b>Attribute Name</b> is the name in which Drupal sends the data in the response whereas in the <b>Attribute Machine Name</b> includes the machine name of the same Drupal attribute.</div>'),
            '#prefix' => '<br><hr><br>',
        );

        $form['oauth_server_basic_attribute_mapping']['mo_oauth_server_attr_list'] = array(
            '#type' => 'table',
            '#header'=> array( 'Attribute Name', 'Attribute Machine Name' ),
            '#empty' => t('Something is not right. Please run the update script or contact us at <a href="mailto:'.MiniorangeOAuthServerConstants::SUPPORT_EMAIL.'">'. MiniorangeOAuthServerConstants::SUPPORT_EMAIL .'</a>'),
            '#responsive' => TRUE ,
            '#prefix' => '<br>',
        );

        $row = self::miniorange_oauth_server_basic_mapping('email', 'mail', false);
        $form['oauth_server_basic_attribute_mapping']['mo_oauth_server_attr_list']['basic_mapping'] = $row;

        $form['oauth_server_custom_attribute_mapping'] = array(
            '#type' => 'fieldset',
            '#title' => t('Custom Mapping <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium"><span class="mo_pro_tooltip">Available in the Premium version</span></a>'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $form['oauth_server_custom_attribute_mapping']['oauth_server_sso_custom_attribute_markup'] = array(
            '#markup' => t('<div class="mo_oauth_server_highlight_background_note_2">In this section, you can send Custom Attributes from Drupal to your OAuth/OIDC Client.</div>'),
            '#prefix' => '<br><hr><br>',
        );

        $form['oauth_server_custom_attribute_mapping']['mo_oauth_server_custom_attr_list'] = array(
            '#type' => 'table',
            '#header'=> array( 'Attribute Name', 'Attribute Machine Name' , '', ''),
            '#empty' => t('Something is not right. Please run the update script or contact us at <a href="mailto:'.MiniorangeOAuthServerConstants::SUPPORT_EMAIL.'">'. MiniorangeOAuthServerConstants::SUPPORT_EMAIL .'</a>'),
            '#responsive' => TRUE ,
            '#prefix' => '<br>',
        );

        $custom_map = self::miniorange_oauth_server_basic_mapping('', '', true);
        $form['oauth_server_custom_attribute_mapping']['mo_oauth_server_custom_attr_list']['custom_mapping'] = $custom_map;

        $form['next_step_1'] = array(
            '#type' => 'submit',
            '#disabled' => true,
            '#value' => t('Save Settings'),
        );
        Utilities::Headless_solution( $form, $form_state);

        Utilities::moOAuthServerShowCustomerSupportIcon($form, $form_state);

        return $form;
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {}

    public function miniorange_oauth_server_basic_mapping($attr_name, $attr_value, $is_custom = false){

        if ($is_custom){
            $row['custom_attr_name'] = [
                '#type' => 'textfield',
                '#attributes' => ['placeholder' => t('Enter Attribute Name')],
                '#disabled' => true,
            ];

            $row['custom_attr_machine_name'] = [
                '#type' => 'textfield',
                '#attributes' => ['placeholder' => t('Enter Attribute Machine Name')],
                '#disabled' => true,
            ];

            $row['mo_plus_button'] = [
                '#type' => 'button',
                '#value' => t('+'),
                '#disabled' => true,
            ];

            $row['mo_minus_button'] = [
                '#type' => 'button',
                '#value' => t('-'),
                '#disabled' => true,
            ];
        }else{
            $row[$attr_name] = [
                '#type' => 'textfield',
                '#default_value' => $attr_name,
                '#attributes' => ['placeholder' => t('Enter Attribute Name')],
                '#disabled' => true,
            ];

            $row[$attr_value] = [
                '#type' => 'textfield',
                '#default_value' => $attr_value,
                '#attributes' => ['placeholder' => t('Enter Attribute Machine Name')],
                '#disabled' => true,
            ];
        }

        return $row;
    }

    /**
     * Send support query.
     */
    function send_support_query(&$form, $form_state)
    {
        Utilities::send_query($form, $form_state);
    }
}
