<?php

namespace Drupal\oauth_server_sso\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\oauth_server_sso\MiniorangeOAuthServerSupport;

class MoOAuthRequestSupport extends FormBase
{
    public function getFormId() {
        return 'oauth_server_sso_request_support';
    }

    public function buildForm(array $form, FormStateInterface $form_state, $options = NULL) {
        $form['#prefix'] = '<div id="modal_support_form">';
        $form['#suffix'] = '</div>';
        $form['status_messages'] = [
            '#type' => 'status_messages',
            '#weight' => -10,
        ];

        $form['markup_library'] = array(
          '#attached' => array(
            'library' => array(
              "oauth_server_sso/oauth_server_sso.style_settings",
            )
          ),
        );

        $user_email = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_email');
        $phone = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_phone');

        $form['markup_1'] = array(
          '#markup' => t('<p class="mo_oauth_server_highlight_background_note_2">Need any help? We can help you with configuring miniOrange OAuth Server module on your site. Just send us a query and we will get back to you soon.</p>'),
        );
        $form['mo_oauth_support_email_address'] = array(
          '#type' => 'email',
          '#required' => true,
          '#title' => t('Email'),
          '#default_value' => $user_email,
          '#attributes' => array('placeholder' => t('Enter your email')),
        );
        $form['mo_oauth_support_phone_number'] = array(
          '#type' => 'textfield',
          '#title' => t('Phone'),
          '#default_value' => $phone,
          '#attributes' => array('placeholder' => t('Enter number with country code Eg. +00xxxxxxxxxx')),
        );
        $form['mo_oauth_support_query'] = array(
          '#type' => 'textarea',
          '#required' => true,
          '#title' => t('Query'),
          '#attributes' => array('placeholder' => t('Describe your query here!')),
        );

        $form['actions'] = array('#type' => 'actions');
        $form['actions']['send'] = [
          '#type' => 'submit',
          '#value' => $this->t('Submit'),
          '#attributes' => [
            'class' => [
              'use-ajax',
              'button--primary'
            ],
          ],
          '#ajax' => [
            'callback' => [$this, 'submitModalFormAjax'],
            'event' => 'click',
          ],
        ];

        $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
        return $form;
    }

    public function submitModalFormAjax(array $form, FormStateInterface $form_state) {
        $response = new AjaxResponse();
        $form_values = $form_state->getValues();
        // If there are any form errors, AJAX replace the form.
        if ( $form_state->hasAnyErrors() ) {
            $response->addCommand(new ReplaceCommand('#modal_support_form', $form));
        } else {
            $email = $form_values['mo_oauth_support_email_address'];
            $phone = $form_values['mo_oauth_support_phone_number'];
            $query = $form_values['mo_oauth_support_query'];
            $query_type = 'Support';

            $support = new MiniorangeOAuthServerSupport($email, $phone, $query, $query_type);
            $support_response = $support->sendSupportQuery();

            if (isset($_SERVER['HTTP_REFERER']) && !empty($_SERVER['HTTP_REFERER'])) {
                global $base_url;
                $current_path = explode($base_url, $_SERVER['HTTP_REFERER']);
                $url_object = \Drupal::service('path.validator')->getUrlIfValid($current_path[1]);
                $route_name = $url_object->getRouteName();
            } else {
                $route_name = 'oauth_server_sso.config_client';
            }

            \Drupal::messenger()->addStatus(t('Support query successfully sent. We will get back to you shortly.'));
            $response->addCommand(new RedirectCommand(Url::fromRoute($route_name)->toString()));
        }
        return $response;
    }

    public function validateForm(array &$form, FormStateInterface $form_state) { }

    public function submitForm(array &$form, FormStateInterface $form_state) { }

}
