<?php
/**
 * @package    miniOrange
 * @subpackage Plugins
 * @license    GNU/GPLv3
 * @copyright  Copyright 2015 miniOrange. All Rights Reserved.
 *
 *
 * This file is part of miniOrange OAuth Server module for Drupal.
 *
 * miniOrange Drupal OAuth Server module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * miniOrange Drupal OAuth Server module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with miniOrange OAuth Server module.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Drupal\oauth_server_sso;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\oauth_server_sso\MiniorangeOAuthServerSupport;

class Utilities {

	public static function isCurlInstalled() {
	    if (in_array('curl', get_loaded_extensions())) {
	        return 1;
	    }
	    else {
	        return 0;
	    }
	}

    public static function Advertise_2fa(array &$form, FormStateInterface $form_state)
    {
        global $base_url;
        $url_path = $base_url . '/' . \Drupal::service('extension.list.module')->getPath('oauth_server_sso'). '/includes/images';

        $download_now = 'https://www.drupal.org/project/miniorange_2fa';
        $know_more = 'https://plugins.miniorange.com/drupal-two-factor-authentication-2fa';

        $form['markup_idp_attr_header_top_support'] = array('#markup' => '</div><div class="mo_oauth_server_table_layout_support_1 mo_oauth_container">',);

        $form['markup_support_1'] = array(
            '#markup' => '<div class="mo_2fa_advertise"><h4>Secure your Drupal login using miniOrange 2FA/MFA</h4> <img class="mo_oauth_2fa_icon" src="' . $url_path . '/2fa_authentication.jpg" alt="2FA Authentication"> <h3>Drupal Second-Factor Authentication</h3><hr></div><br>',
        );

        $form['oauthserver_2fa_description'] = array(
          '#markup' => t('<div class="mo_oauth_2fa_describe"> Two Factor Authentication (2FA) module adds a second layer of authentication at the time of login to secure your Drupal accounts. Also, Two-Factor Authentication (2FA) module provides 17+ 2fa methods including OTP Over SMS, Email, Phone call, WhatsApp, Google Authenticator, etc.</div><br>'),
        );

        $form['oauth_server_2fa_download'] = array(
            '#type' => 'link',
            '#title' => t('Download Now'),
            '#url' => Url::fromUri($download_now),
            '#attributes' => ['class' => ['button', 'button--primary'], 'target' => '_blank'],
            '#prefix' => '<div class="mo_2fa_advertise">'
        );

        $form['oauth_server_2fa_know_more'] = array(
            '#type' => 'link',
            '#title' => t('Know More'),
            '#url' => Url::fromUri($know_more),
            '#attributes' => ['class' => ['button', 'button--primary'], 'target' => '_blank'],
            '#suffix' => '</div>'
        );
    }

    public static function Headless_solution(array &$form, FormStateInterface $form_state)
    {
        global $base_url;
        $url_path = $base_url . '/' . \Drupal::service('extension.list.module')->getPath('oauth_server_sso'). '/includes/images';

        $know_more = 'https://plugins.miniorange.com/decoupled-headless-drupal';

        $form['markup_top_headless'] = array('#markup' => '</div><div class="mo_oauth_server_table_layout_support_1 mo_oauth_container">',);

        $form['markup_support_1'] = array(
            '#markup' => '<div class="mo_2fa_advertise"><h4>Enable Single Sign On for your Headless / Decoupled Drupal site</h4> <br><img class="mo_oauth_headless_icon" src="' . $url_path . '/headless_sso.png" alt="Headless SSO"> <h3>Headless Drupal Single Sign-On</h3><hr></div><br>',
        );

        $form['oauthserver_2fa_description'] = array(
          '#markup' => t('<div class="mo_oauth_2fa_describe"> Enable Single Sign On for your Headless / Decoupled Drupal site using any of the popular protocols such as SAML, OAuth 2.0, OpenID Connect (OIDC), etc. Our Headless Drupal SSO module is compatible with most popular Front End Technologies like ReactJS, AngularJS, NestJS, VueJS right out of the box.</div><br>'),
        );

        $form['oauth_server_2fa_know_more'] = array(
            '#type' => 'link',
            '#title' => t('Know More'),
            '#url' => Url::fromUri($know_more),
            '#attributes' => ['class' => ['button', 'button--primary'], 'target' => '_blank'],
            '#prefix' => '<div class="mo_2fa_advertise">',
            '#suffix' => '</div>'
        );
    }

    public static function send_query(&$form, $form_state)
    {

        $email = $form['oauth_server_sso_email_address']['#value'];
        $phone = $form['oauth_server_sso_phone_number']['#value'];
        $query = $form['oauth_server_sso_support_query']['#value'];
        if(empty($email)||empty($query)){
            \Drupal::messenger()->addError(t('The <b><u>Email</u></b> and <b><u>Query</u></b> fields are mandatory.'));
            return;
        } elseif(!(\Drupal::service('email.validator')->isValid($email))) {
            \Drupal::messenger()->addError(t('The email address <b><i>' . $email . '</i></b> is not valid.'));
            return;
        }
        $support = new MiniorangeOAuthServerSupport($email, $phone, $query);
        $support_response = $support->sendSupportQuery();
        if($support_response) {
            \Drupal::messenger()->addMessage(t('Support query successfully sent'));
        }
        else {
            \Drupal::messenger()->addError(t('Error sending support query'));
        }
    }

    public static function showErrorMessage( $error, $message, $cause, $closeWindow = FALSE ) {
      global $base_url;
      $actionToTakeUponWindow = $closeWindow === TRUE ? 'onClick="self.close();"' : 'href="' . $base_url . '/user/login"';
      echo '<div style="font-family:Calibri;padding:0 3%;">';
      echo '<div style="color: #a94442;background-color: #f2dede;padding: 15px;margin-bottom: 20px;text-align:center;border:1px solid #E6B3B2;font-size:18pt;"> ERROR</div>
                                    <div style="color: #a94442;font-size:14pt; margin-bottom:20px;"><p><strong>Error: </strong>' .  $error  . '</p>                                      <p>' .  $message  . '</p>
                                        <p><strong>Possible Cause: </strong>' .  $cause  . '</p>
                                    </div>'
                                    ;
      exit;
    }

    public static function drupal_is_cli()
    {
      $server = \Drupal::request()->server;
      $server_software = $server->get('SERVER_SOFTWARE');
      $server_argc = $server->get('argc');

      return !isset($server_software) && (php_sapi_name() == 'cli' || (is_numeric($server_argc) && $server_argc > 0)) ? true : false;
    }

    public static function mo_get_drupal_core_version() {
      return \DRUPAL::VERSION;
    }

  public static function moOAuthServerShowCustomerSupportIcon(array &$form, FormStateInterface $form_state){
    global $base_url;
    $support_image_path = $base_url . '/' . \Drupal::service('extension.list.module')->getPath('oauth_server_sso'). '/includes/images';
    $form['mo_oauth_client_customer_support_icon'] = [
      '#markup' => t('<a class="use-ajax mo-bottom-corner" href="requestSupport"><img src="' . $support_image_path . '/mo-customer-support.png" alt="test image"></a>'),
    ];
  }

}
?>
